
import { Webcomponent } from './webcomponent.js';

const config = {
    content: {
        wrapper: ".wrapper",
        list: ".item-list"
    },
    lang: {
        create: "Create «{$name}»"
    }
};

class UiSelect extends Webcomponent {
    constructor() {
        super();

        this.fetch_template();

        this.wrapper = this.shadowRoot.querySelector(config.content.wrapper);
        this.list = this.shadowRoot.querySelector(config.content.list);
        this.input = this.querySelector('input:not([type="hidden"])');
        this.select = this.querySelector('select');
        this.active = false;
        
        this.init();
    }

    init() {
        this.input.setAttribute('readonly', 'readonly');

        if ( this.getAttribute("create") !== null ) {
            this.item_create = this.add_item("__create__", config.lang.create, false, true);
        }

        Array.from(this.select.options).forEach(function(item) {
            this.add_item(item.value, item.text, item.value === this.select.value);
        }.bind(this));

        [ "mouseover" , "mouseleave" ].forEach(function(ev) {
            this.list.addEventListener(ev, function(e) {
                this.active = ( ev === 'mouseover' ) && ( document.activeElement === this.input );
            }.bind(this));
        }.bind(this));

        [ "focus" , "blur", "keyup", "click" ].forEach(function(ev) {
            this.input.addEventListener(ev, function(e) {
                switch(ev) {
                    case "focus":
                    case "blur":
                        setTimeout(function() {
                            this.has_focus = ev === "focus";
                            this[ (this.has_focus ? "show" : "hide" ) + "_list" ].call(this, false);
                        }.bind(this), 100);
                    break;

                    case "click":
                        if ( ! this.editing && this.has_focus ) {
                            this.editing = true;
                            this.input.removeAttribute('readonly');
                            this.show_list();
                        }
                    break;

                    case "keyup":
                        e.target.value && this.editing ? this.filter(e.target.value) : this.show_list(true);
                    break;
                }

            }.bind(this));
        }.bind(this));
    }

    add_item(key, value, selselfected, hide) {
        let elem = document.createElement('li');

        if (this.list.querySelector(`[data-key="${key}"]`) !== null) {
            console.error(`Item with key '${key}' is already existing.`);
            return false;
        }

        elem.setAttribute('data-key', key);

        if (! (elem.innerText = value) ) {
            elem.innerHTML = "&nbsp;";
        }

        let self = this;

        elem.addEventListener("click", function(e) {
            self.set_value(this.getAttribute('data-key'), this.innerText);

            let item = self.list.querySelector(".selected");

            if ( item ) {
                item.classList.remove("selected");
            }

            this.classList.add("selected");

            self.hide_list(true);
        });

        if (selected) {
            elem.click();
            // this.input.key_up();
        }

        if (hide) {
            elem.classList.add('hidden');
        }

        this.list.appendChild(
            elem
        );

        return elem;
    }

    set_value(key, value) {
        this.input.value = value.trim();
        this.select.value = key;
    }

    hide_list(force) {
        if ( ! this.active || force ) {
            this.has_focus = this.editing = false;
            this.list.classList.add('hidden');
            this.input.setAttribute('readonly', 'readonly');
            this.show_all_items();
            this.item_create.classList.add('hidden');
        }
    }

    show_list(show_all_items) {
        this.list.classList.remove('hidden');

        if ( show_all_items ) {
            this.show_all_items();
        }
        else {
            this.editing && this.input.value && this.filter(this.input.value);
        }

        if ((! this.querySelectorAll(":not(.hidden)").length || show_all_items ) && this.item_create ) {
            this.item_create.classList.add('hidden');
        }
    }

    flush() {

    }

    filter(org_query) {
        let empty = true,
            has_exact_result = false,
            query = org_query.trim().normalize().toLowerCase();

        Array.from( this.list.querySelectorAll('li:not([data-key="__create__"])') ).forEach(function(item) {
            let text = item.innerText.normalize().toLowerCase(),
                keep = false;

            if ( text === query ) {
                keep = true;
                empty = false;
                has_exact_result = true;
            }
            else {
                query.split(this._advanced_search ? " " : "\n").filter(item => item.length > 1).forEach(function(query_item) {
                    if ( text.indexOf(query_item) !== -1 ) {
                        keep = true;
                        empty = false;
                    }
                });
            }

            if ( keep ) {;
                item.classList.remove('hidden');
            }
            else {
                item.classList.add('hidden');
            }
        }.bind(this));

        if ( this.item_create ) {
            if ( empty && ! has_exact_result ) {
                this.item_create.classList.remove('hidden');

                this.item_create.innerText = this.replace_vars(config.lang.create, {
                    name: org_query
                });
            }
            else {
                this.item_create.classList.add('hidden');
            }
        }
    }

    replace_vars(content, vars) {
        return content.replace(/{\$(.*?)}/g, function(match, varname) {
            return vars[varname];
        });
    }

    show_all_items() {
        Array.from( this.list.querySelectorAll('li') ).forEach(item => item.classList.remove('hidden'));
    }

    get content() {
        return this._content;
    }

    set content(value) {
        return this._content = value;
    }

    get list() {
        return this._list;
    }

    set list(value) {
        return this._list = value;
    }

    get wrapper() {
        return this._wrapper;
    }

    set wrapper(value) {
        return this._wrapper = value;
    }

    get input() {
        return this._input;
    }

    set input(value) {
        return this._input = value;
    }

    get active() {
        return this._active;
    }

    set active(value) {
        return this._active = value;
    }

    get item_create() {
        return this._item_create;
    }

    set item_create(value) {
        return this._item_create = value;
    }

    get has_focus() {
        return this._has_focus;
    }

    set has_focus(value) {
        return this._has_focus = value;
    }

    get editing() {
        return this._editing;
    }

    set editing(value) {
        this.wrapper.classList[value ? "add" : "remove"]('is-editing');
        return this._editing = value;
    }

    get advanced_search() {
        return this._advanced_search;
    }

    set advanced_search(value) {
        return this._advanced_search = value;
    }
}

export { UiSelect }
